part of dartvolt;

class Client {
    Client(ClientConfig config) {
        this.clientConfig = config;
    }
    
    ClientConfig clientConfig = ClientConfig();
    late ServerConfig serverConfig;
    late WSClient wsClient;
    Map<String, String> _authHeaders = {};
    var _authStarted = false;
    SessionInfo? sessionInfo;
    
    late var users = UserManager(this);
    late var channels = ChannelManager(this);
    
    late final _logger = Logger(this);
    
    /// Surprise surprise, this emits events
    EventEmitter events = EventEmitter();
    
    /// Generate a new session using username/password
    Future<void> login(AuthInfo authInfo) async {
        serverConfig = await _fetchServerConfig();
        //if (serverConfig.captcha != null && authInfo.captcha == null) {
        //    throw 'Server requires captcha, but none was provided';
        //}
        
        var authBody = {
            'email': authInfo.email,
            'password': authInfo.password,
        };
        
        if (authInfo.device_name != null) {
            authBody['device_name'] = authInfo.device_name as String;
        }
        if (authInfo.captcha != null) {
            authBody['captcha'] = authInfo.captcha as String;
        }
        
        var res = await http.post(
            Uri.parse(clientConfig.apiUrl + '/auth/login'),
            body: jsonEncode(authBody),
            headers: { 'User-Agent': clientConfig.user_agent }
        );
        
        print(res.body);
        if (res.statusCode != 200) throw res.body;
    }
    
    /// Use an existing session
    Future<void> useExistingSession(SessionInfo session) async {
        if (_authStarted) throw 'Cannot authenticate again';
        _authStarted = true;
        sessionInfo = session;
        _authHeaders = {
            'x-user-id': session.clientId,
            'x-session-token': session.sessionToken,
            'User-Agent': clientConfig.user_agent,
        };
        
        serverConfig = await _fetchServerConfig();
        
        if (!await _validateSession(session, clientConfig)) {
            throw 'Invalid session';
        }
        
        wsClient = WSClient(revoltClient: this, heartbeat: 10);
        await wsClient.connect();
    }
    
    Future<bool> _validateSession(SessionInfo session, ClientConfig clientConfig) async {
        var res = await http.get(
            Uri.parse(clientConfig.apiUrl + '/auth/check'),
            headers: _authHeaders
        );
        
        return res.statusCode == 200;
    }
    
    Future<ServerConfig> _fetchServerConfig() async {
        var res = await http.get(Uri.parse(clientConfig.apiUrl));
        Map<String, dynamic> config = jsonDecode(res.body);
        Map<String, dynamic> features = config['features'];
        return ServerConfig(
            wsURL:                  config['ws'],
            appURL:                 config['app'],
            vapid:                  config['vapid'],
            version:                config['revolt'],
            emailEnabled:           features['email'],
            inviteOnly:             features['invite_only'],
            registrationEnabled:    features['registration'],
            captcha:                features['captcha']?['key'],
            januaryURL:             features['january']?['url'],
            autumnURL:              features['autumn']?['url'],
            vosoURL:                features['voso']['url'],
            vosoWS:                 features['voso']['ws']
        );
    }
}